/*******************************************************************************
*                                                                              *
* System      : SiTCP                                                          *
* Block       : SIO                                                            *
* Module      : SIO_REG                                                        *
* Version     : v 0.0.0 2008/03/26 12:01                                       *
*                                                                              *
* Description : A sample for the Serial IO interface                           *
*                                                                              *
* Designer    : Tomohisa Uchida                                                *
*                                                                              *
*                Copyright (c) 2008 Tomohisa Uchida                            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/
module SIO_REG(
	SCK					,	// in	: Clock
	RSTn				,	// in	: System reset
	//test by hirose
	ADDR00				,
	ADDR01				,
	ADDR02				,
	ADDR03				,
	ADDR04				,
	ADDR05				,
	ADDR06				,
	ADDR07				,
	ADDR08				,
	ADDR09				,
	ADDR0a				,
	ADDR0b				,
	ADDR0c				,

	// Register I/F
	REG_ADDR			,	// in	: Address[31:0]
	REG_WD				,	// in	: Data[7:0]
	REG_WE				,	// in	: Write enable
	REG_RE				,	// in	: Read enable
	REG_ACK				,	// out	: Access acknowledge
	REG_RV				,	// out	: Read valid
	REG_RD					// out	: Read data[7:0]

);

//-------- Input/Output -------------
	input			SCK				;
	input			RSTn			;

	input	[31:0]	REG_ADDR		;
	input	[7:0]	REG_WD			;
	input			REG_WE			;
	input			REG_RE			;
	output			REG_ACK			;
	output			REG_RV			;
	output	[7:0]	REG_RD			;
	output	[7:0]	ADDR00				;
	output	[7:0]	ADDR01				;
	output	[7:0]	ADDR02				;
	output	[7:0]	ADDR03				;
	output	[7:0]	ADDR04				;
	output	[7:0]	ADDR05				;
	output	[7:0]	ADDR06				;
	output	[7:0]	ADDR07				;
	output	[7:0]	ADDR08				;
	output	[7:0]	ADDR09				;
	output	[7:0]	ADDR0a				;
	output	[7:0]	ADDR0b				;
	output	[7:0]	ADDR0c				;

//---------- Buffer ----------
	wire			REG_ACK			;
	wire			REG_RV			;
	wire	[7:0]	REG_RD			;

//------------------------------------------------------------------------------
//	Receive
//------------------------------------------------------------------------------
	wire			regCs			;

	assign	regCs	= (REG_ADDR[31:8]==28'd0);

	reg		[7:0]	x00_Reg			;
	reg		[7:0]	x01_Reg			;
	reg		[7:0]	x02_Reg			;
	reg		[7:0]	x03_Reg			;
	reg		[7:0]	x04_Reg			;
	reg		[7:0]	x05_Reg			;
	reg		[7:0]	x06_Reg			;
	reg		[7:0]	x07_Reg			;
	reg		[7:0]	x08_Reg			;
	reg		[7:0]	x09_Reg			;
	reg		[7:0]	x0A_Reg			;
	reg		[7:0]	x0B_Reg			;
	reg		[7:0]	x0C_Reg			;
	reg		[7:0]	x0D_Reg			;
	reg		[7:0]	x0E_Reg			;
	reg		[7:0]	x0F_Reg			;

	always@ (posedge SCK or negedge RSTn) begin
		if(~RSTn)begin
			x00_Reg[7:0]	<= 8'd0;
			x01_Reg[7:0]	<= 8'd0;
			x02_Reg[7:0]	<= 8'd0;
			x03_Reg[7:0]	<= 8'd0;
			x04_Reg[7:0]	<= 8'd0;
			x05_Reg[7:0]	<= 8'd0;
			x06_Reg[7:0]	<= 8'd0;
			x07_Reg[7:0]	<= 8'd0;

			x08_Reg[7:0]	<= 8'd0;
			x09_Reg[7:0]	<= 8'd0;
			x0A_Reg[7:0]	<= 8'd0;
			x0B_Reg[7:0]	<= 8'd0;
			x0C_Reg[7:0]	<= 8'd0;
			x0D_Reg[7:0]	<= 8'd0;
			x0E_Reg[7:0]	<= 8'd0;
			x0F_Reg[7:0]	<= 8'd0;
		end else begin
			if(REG_WE)begin
				x00_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h0) ? REG_WD[7:0] : x00_Reg[7:0]);
				x01_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h1) ? REG_WD[7:0] : x01_Reg[7:0]);
				x02_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h2) ? REG_WD[7:0] : x02_Reg[7:0]);
				x03_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h3) ? REG_WD[7:0] : x03_Reg[7:0]);
				x04_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h4) ? REG_WD[7:0] : x04_Reg[7:0]);
				x05_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h5) ? REG_WD[7:0] : x05_Reg[7:0]);
				x06_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h6) ? REG_WD[7:0] : x06_Reg[7:0]);
				x07_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h7) ? REG_WD[7:0] : x07_Reg[7:0]);
				x08_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h8) ? REG_WD[7:0] : x08_Reg[7:0]);
				x09_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'h9) ? REG_WD[7:0] : x09_Reg[7:0]);
				x0A_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'hA) ? REG_WD[7:0] : x0A_Reg[7:0]);
				x0B_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'hB) ? REG_WD[7:0] : x0B_Reg[7:0]);
				x0C_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'hC) ? REG_WD[7:0] : x0C_Reg[7:0]);
				x0D_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'hD) ? REG_WD[7:0] : x0D_Reg[7:0]);
				x0E_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'hE) ? REG_WD[7:0] : x0E_Reg[7:0]);
				x0F_Reg[7:0]	<= (regCs & (REG_ADDR[3:0]==4'hF) ? REG_WD[7:0] : x0F_Reg[7:0]);
			end
		end
	end

	reg		[7:0]	rdData		;
	reg				orRv		;
	reg				orAck		;

	always@ (posedge SCK) begin
		case(REG_ADDR[3:0])
			4'h0:	rdData[7:0]	<= x00_Reg[7:0];
			4'h1:	rdData[7:0]	<= x01_Reg[7:0];
			4'h2:	rdData[7:0]	<= x02_Reg[7:0];
			4'h3:	rdData[7:0]	<= x03_Reg[7:0];
			4'h4:	rdData[7:0]	<= x04_Reg[7:0];
			4'h5:	rdData[7:0]	<= x05_Reg[7:0];
			4'h6:	rdData[7:0]	<= x06_Reg[7:0];
			4'h7:	rdData[7:0]	<= x07_Reg[7:0];
			4'h8:	rdData[7:0]	<= x08_Reg[7:0];
			4'h9:	rdData[7:0]	<= x09_Reg[7:0];
			4'hA:	rdData[7:0]	<= x0A_Reg[7:0];
			4'hB:	rdData[7:0]	<= x0B_Reg[7:0];
			4'hC:	rdData[7:0]	<= x0C_Reg[7:0];
			4'hD:	rdData[7:0]	<= x0D_Reg[7:0];
			4'hE:	rdData[7:0]	<= x0E_Reg[7:0];
			default:rdData[7:0]	<= x0F_Reg[7:0];
		endcase
		orRv	<= REG_RE;
		orAck	<= REG_WE | REG_RE;
	end

	assign	REG_ACK		= orAck;
	assign	REG_RV		= orRv;
	assign	REG_RD[7:0] = (orRv ? rdData[7:0] : 8'd0);

	assign  	ADDR00 = x00_Reg;
	assign  	ADDR01 = x01_Reg;
	assign  	ADDR02 = x02_Reg;
	assign  	ADDR03 = x03_Reg;
	assign  	ADDR04 = x04_Reg;
	assign  	ADDR05 = x05_Reg;
	assign  	ADDR06 = x06_Reg;
	assign  	ADDR07 = x07_Reg;
	assign  	ADDR08 = x08_Reg;
	assign  	ADDR09 = x09_Reg;
	assign  	ADDR0a = x0A_Reg;
	assign  	ADDR0b = x0B_Reg;
	assign  	ADDR0c = x0C_Reg;
//------------------------------------------------------------------------------
endmodule
